import Flutter
import UIKit
import PaygilantSDK
import CoreLocation

public class SwiftPaygilantPlugin: NSObject, FlutterPlugin {

    var screenListener : PaygilantScreenListener? = nil
    
  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "paygilant_plugin", binaryMessenger: registrar.messenger())
    let instance = SwiftPaygilantPlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)
  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {

    switch call.method {
    case "getPlatformVersion":
            result("iOS " + UIDevice.current.systemVersion)
        break
    case "init":
        guard
            let dataParser = call.arguments as? [String: Any],
            let serverUrl = dataParser["serverUrl"] as? String
            else{
                result(FlutterError(code: "Error arguments",  message: "Cant add arguments to init", details: nil))
                return
        }
        let userId = dataParser["userId"] as? String
        do{
            try PaygilantManager.shared.setup(serverUrl: serverUrl, userId: userId)
        }catch let error as PaygilantError {
            print(error.description)
        }
        catch{}
        break
        
    case "getDeviceId":
            result(PaygilantManager.shared.getDeviceId())
        break
    case "getSessionId":
            result(PaygilantManager.shared.getSessionId())
        break
    case "initializeDeviceId":
        PaygilantManager.shared.initializeDeviceId()
      break
    case "logout":
      PaygilantManager.shared.logout()

      break
    case "startNewScreenListener":
        guard
            let dataParser = call.arguments as? [String: Any],
            let view = UIApplication.shared.keyWindow?.rootViewController?.view,
            let screenListenerType = dataParser["screenListenerType"] as? String,
            let type = ScreenListenerTypeToString.fromString(nillableString:screenListenerType),
            let screenId = dataParser["screenId"] as? Int
        else{
                result(FlutterError(code: "Error arguments",  message: "Cant add arguments to screenListener", details: nil))
                return
            }
        self.screenListener = PaygilantManager.shared.startNewScreenListener(with: type, actionId: screenId, view: view)
      break;
    case "pauseListenToSensors":
        self.screenListener?.finishScreenListener()
        break
    case "arriveToCheckPoint":
        guard
            let dataParser = call.arguments as? [String: Any],
            let strCheckPoint = dataParser["checkpoint"] as? String,
            let dict  = convertToDictionary(text: strCheckPoint),
            let checkPoint = try? CheckPoint.parse(dict)
        else {
            result(FlutterError(code: "Error arguments",  message: "Cant add arguments to arriveToCheckPoint", details: nil))
            return
        }
        PaygilantManager.shared.arriveToCheckPoint(checkPoint: checkPoint)
        break
        case "getRiskForCheckPoint":
            guard
                let dataParser = call.arguments as? [String: Any],
                let jsonStr =  dataParser["checkpoint"] as? String,
                let dict  = convertToDictionary(text: jsonStr),
                let checkPoint = try? CheckPoint.parse(dict)
            else {
                result(FlutterError(code: "Error arguments",  message: "Cant add arguments to arriveToCheckPoint", details: nil))
                return
            }
            PaygilantManager.shared.getRiskForCheckPoint(checkPoint: checkPoint) { (risk, signRisk, requestId) in
                var json = [String:Any]()
                json["riskLevel"] = risk.riskLevel.rawValue
                json["signedRisk"] = signRisk
                json["requestId"] = requestId
                result(self.jsonToString(json: json))
            }
        break
    case "updateCheckPointStatus":
        guard
            let options = call.arguments as? [String: Any],
            let checkPointTypeStr = options["checkPointType"] as? String,
            let  checkPointType = CheckPointTypeToString.fromString(nillableString: checkPointTypeStr),
            let requestID = options["requestID"] as? String,
            let checkPointStatusStr = options["checkPointStatus"] as? String,
            let transactionID = options["transactionID"] as? String
            else{
                result(FlutterError(code: "Error arguments",  message: "Cant add arguments to screenListener", details: nil))
                return
        }
        let checkPointStatus = CheckPointStatusToString.fromString(string: checkPointStatusStr)
        PaygilantManager.shared.updateCheckPointStatus(type: checkPointType, requestID: requestID, status: checkPointStatus, transactionID: transactionID)
        break
    case "onRequestPermissionsResult":
        guard
            let options = call.arguments as? [String: Any],
            let permissions = options["permissions"] as? [String]
            else{
                result(FlutterError(code: "Error arguments",  message: "Cant add arguments to onRequestPermissionsResult", details: nil))
                return
        }
        for permission in permissions{
            if let typeMobilePermission =  parseMobilePermission(permission){
                PaygilantManager.shared.onRequestAccess(typeMobilePermission: typeMobilePermission)
            }
        }
        break
    default: break
    }
  }
    func jsonToString(json: [String:Any]) -> String{
        do {
            let data1 = try JSONSerialization.data(withJSONObject: json, options: JSONSerialization.WritingOptions.prettyPrinted)
            let convertedString = String(data: data1, encoding: String.Encoding.utf8) as NSString? ?? ""
            debugPrint(convertedString)
            return convertedString as String
        } catch let myJSONError {
            debugPrint(myJSONError)
            return ""
        }
    }
    func convertToDictionary(text: String) -> [String: Any]? {
        if let data = text.data(using: .utf8) {
            do {
                return try JSONSerialization.jsonObject(with: data, options: []) as? [String: Any]
            } catch {
                print(error.localizedDescription)
            }
        }
        return nil
    }
    func parseMobilePermission(_ value: String) -> MobilePermission? {
        switch value {
            case "contacts":
                return .readContacts
            case "locationWhenInUse":
                return .authorizedWhenInUse
            case "locationAlways":
                return .authorizedAlways
        case "location":
            let status = CLLocationManager.authorizationStatus()
            if status == CLAuthorizationStatus.authorizedAlways {
                return .authorizedAlways
            }else{
                return .authorizedWhenInUse
            }
            case "photos":
                return .photo
            case "camera":
                return .camera
            case "reminders":
                return .reminder
            case "calendar":
                return .calendar
            default:
            return nil
        }
    }
}

extension CheckPoint {
    static func parse(_ object: Any?) throws -> CheckPoint {
        guard
            let json = object as? [String: Any],
            let typeString = json["checkpoint_type"] as? String,
            let type = CheckPointTypeToString.fromString(nillableString: typeString)
        else { throw NSError(domain: "", code: 0, userInfo: [:]) }
        
        switch type {
        case .LOGIN:
                return Login(json: json)
        case .REGISTER:
                return Register(json: json)
        case .ADD_PAYMENT_METHOD:
            return AddPaymentMethod(json: json)
        case .GENERAL:
            return General(json: json)
        case .TRANSACTION:
            return Transaction(json: json)
        case .LAUNCH:
            return Launch(json: json)
        case .UPDATE_USER_INFO:
            return UpdateUserInfo(json: json)
        case .PASSWORD_CHANGE:
            return PasswordChange(json: json)
        case .VIEW_CARD_DETAILS:
            return ViewCardDetails(json: json)
        default:
            throw NSError(domain: "", code: 0, userInfo: [:])
        }
    }
}

